<?php
/**
 * Copyright (C) 2011-2015 by Lars Strojny <lstrojny@php.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
namespace Functional;

use Functional\Exceptions\InvalidArgumentException;
use Traversable;

/**
 * Recombines arrays by index and applies a callback optionally
 *
 * @param Traversable[]|array[] $collections One or more collections, last argument may be a callback
 * @return array
 */
function zip(...$collections)
{
    $callback = null;
    if (is_callable(end($collections))) {
        $callback = array_pop($collections);
    }

    foreach ($collections as $position => $arg) {
        InvalidArgumentException::assertCollection($arg, __FUNCTION__, $position + 1);
    }

    $firstCollection = reset($collections);
    $result = [];
    if ($callback === null) {
        foreach ($firstCollection as $index => $value) {
            $zipped = [];

            foreach ($collections as $arg) {
                $zipped[] = isset($arg[$index]) ? $arg[$index] : null;
            }

            $result[] = $zipped;
        }
    } else {
        foreach ($firstCollection as $index => $value) {
            $zipped = [];

            foreach ($collections as $arg) {
                $zipped[] = isset($arg[$index]) ? $arg[$index] : null;
            }

            $result[] = $callback(...$zipped);
        }
    }

    return $result;
}
